@@ -120,12 +120,19 @@ void		gedit_document_insert_text	(GeditDocument *doc,
 						 gint	        pos,
                                                  const gchar   *text,
                                                  gint           len);
+
+void		gedit_document_insert_text_at_cursor (GeditDocument *doc, 
+                                                 const gchar   *text,
+                                                 gint           len);
+
 void 		gedit_document_delete_text 	(GeditDocument *doc, 
 		                                 gint start, gint end);
 
 gchar*		gedit_document_get_chars 	(GeditDocument *doc, 
 					         gint start, gint end);
 
+gchar*		gedit_document_get_selected_text (GeditDocument *doc);
+gboolean	gedit_document_has_selected_text (GeditDocument *doc);
 
 /* Multi-level Undo/Redo operations */
 gboolean	gedit_document_can_undo		(const GeditDocument *doc);
@@ -137,12 +144,25 @@ void		gedit_document_redo 		(GeditDocument *doc);
 void		gedit_document_begin_not_undoable_action 	(GeditDocument *doc);
 void		gedit_document_end_not_undoable_action		(GeditDocument *doc);
 
+void		gedit_document_begin_user_action (GeditDocument *doc);
+void		gedit_document_end_user_action	 (GeditDocument *doc);
+
 void		gedit_document_goto_line 	(GeditDocument* doc, guint line);
+
 gchar* 		gedit_document_get_last_searched_text (GeditDocument* doc);
+gchar* 		gedit_document_get_last_replace_text  (GeditDocument* doc);
+
 gboolean	gedit_document_find 		(GeditDocument* doc, const gchar* str, 
 						 gboolean from_cursor, 
 						 gboolean case_sensitive);
 gboolean	gedit_document_find_again	(GeditDocument* doc);
+
+void		gedit_document_replace_selected_text (GeditDocument *doc, 
+						      const gchar *replace);
+gboolean	gedit_document_replace_all (GeditDocument *doc,
+				            const gchar *find, const gchar *replace, 
+					    gboolean case_sensitive);
+
 #endif /* __GEDIT_DOCUMENT_H__ */
 
 
